@@ -47,7 +47,7 @@ class HtmlTestHelpers(CoverageTest):
         self.clean_local_file_imports()
         cov = coverage.Coverage(**(covargs or {}))
         self.start_import_stop(cov, "main_file")
-        cov.html_report(**(htmlargs or {}))
+        return cov.html_report(**(htmlargs or {}))
 
     def remove_html_files(self):
         """Remove the HTML files created as part of the HTML report."""
@@ -102,11 +102,7 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         # At least one of our tests monkey-patches the version of coverage.py,
         # so grab it here to restore it later.
         self.real_coverage_version = coverage.__version__
-        self.addCleanup(self.cleanup_coverage_version)
-
-    def cleanup_coverage_version(self):
-        """A cleanup."""
-        coverage.__version__ = self.real_coverage_version
+        self.addCleanup(setattr, coverage, "__version__", self.real_coverage_version)
 
     def test_html_created(self):
         # Test basic HTML generation: files should be created.
@@ -209,6 +205,23 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         fixed_index2 = index2.replace("XYZZY", self.real_coverage_version)
         self.assertMultiLineEqual(index1, fixed_index2)
 
+    def test_file_becomes_100(self):
+        self.create_initial_files()
+        self.run_coverage()
+
+        # Now change a file and do it again
+        self.make_file("main_file.py", """\
+            import helper1, helper2
+            # helper1 is now 100%
+            helper1.func1(12)
+            helper1.func1(23)
+            """)
+
+        self.run_coverage(htmlargs=dict(skip_covered=True))
+
+        # The 100% file, skipped, shouldn't be here.
+        self.assert_doesnt_exist("htmlcov/helper1_py.html")
+
     def test_status_format_change(self):
         self.create_initial_files()
         self.run_coverage()
@@ -282,18 +295,20 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
     """Test the behavior when measuring unparsable files."""
 
     def test_dotpy_not_python(self):
+        self.make_file("main.py", "import innocuous")
         self.make_file("innocuous.py", "a = 1")
         cov = coverage.Coverage()
-        self.start_import_stop(cov, "innocuous")
+        self.start_import_stop(cov, "main")
         self.make_file("innocuous.py", "<h1>This isn't python!</h1>")
         msg = "Couldn't parse '.*innocuous.py' as Python source: .* at line 1"
         with self.assertRaisesRegex(NotPython, msg):
             cov.html_report()
 
     def test_dotpy_not_python_ignored(self):
+        self.make_file("main.py", "import innocuous")
         self.make_file("innocuous.py", "a = 2")
         cov = coverage.Coverage()
-        self.start_import_stop(cov, "innocuous")
+        self.start_import_stop(cov, "main")
         self.make_file("innocuous.py", "<h1>This isn't python!</h1>")
         cov.html_report(ignore_errors=True)
         self.assertEqual(
@@ -359,7 +374,7 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         self.make_file("main.py", "import sub.not_ascii")
         self.make_file("sub/__init__.py")
         self.make_file("sub/not_ascii.py", """\
-            # coding: utf8
+            # coding: utf-8
             a = 1  # Isn't this great?!
             """)
         cov = coverage.Coverage()
@@ -368,7 +383,7 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         # Create the undecodable version of the file. make_file is too helpful,
         # so get down and dirty with bytes.
         with open("sub/not_ascii.py", "wb") as f:
-            f.write(b"# coding: utf8\na = 1  # Isn't this great?\xcb!\n")
+            f.write(b"# coding: utf-8\na = 1  # Isn't this great?\xcb!\n")
 
         with open("sub/not_ascii.py", "rb") as f:
             undecodable = f.read()
@@ -464,6 +479,16 @@ class HtmlTest(HtmlTestHelpers, CoverageTest):
         self.assert_doesnt_exist("htmlcov/main_file_py.html")
         self.assert_exists("htmlcov/not_covered_py.html")
 
+    def test_report_skip_covered_100(self):
+        self.make_file("main_file.py", """
+            def normal():
+                print("z")
+            normal()
+        """)
+        res = self.run_coverage(covargs=dict(source="."), htmlargs=dict(skip_covered=True))
+        self.assertEqual(res, 100.0)
+        self.assert_doesnt_exist("htmlcov/main_file_py.html")
+
     def test_report_skip_covered_branches(self):
         self.make_file("main_file.py", """
             import not_covered
@@ -487,12 +512,8 @@ class HtmlStaticFileTest(CoverageTest):
 
     def setUp(self):
         super(HtmlStaticFileTest, self).setUp()
-        self.original_path = list(coverage.html.STATIC_PATH)
-        self.addCleanup(self.cleanup_static_path)
-
-    def cleanup_static_path(self):
-        """A cleanup."""
-        coverage.html.STATIC_PATH = self.original_path
+        original_path = list(coverage.html.STATIC_PATH)
+        self.addCleanup(setattr, coverage.html, 'STATIC_PATH', original_path)
 
     def test_copying_static_files_from_system(self):
         # Make a new place for static files.
@@ -751,13 +772,12 @@ class HtmlGoldTests(CoverageGoldTest):
         compare("gold_partial", "out/partial", size_within=10, file_pattern="*.html")
         contains(
             "out/partial/partial_py.html",
-            '<p id="t8" class="stm run hide_run">',
+            '<p id="t8" class="stm par run hide_run">',
             '<p id="t11" class="stm run hide_run">',
-            '<p id="t14" class="stm run hide_run">',
             # The "if 0" and "if 1" statements are optimized away.
-            '<p id="t17" class="pln">',
+            '<p id="t14" class="pln">',
             # The "raise AssertionError" is excluded by regex in the .ini.
-            '<p id="t24" class="exc">',
+            '<p id="t21" class="exc">',
         )
         contains(
             "out/partial/index.html",
@@ -765,7 +785,7 @@ class HtmlGoldTests(CoverageGoldTest):
         )
         contains(
             "out/partial/index.html",
-            '<span class="pc_cov">100%</span>'
+            '<span class="pc_cov">91%</span>'
         )
 
     def test_styled(self):
